/*
 Alarm System v3.0 Door
 Version: Test
 Author: Alex from insideGadgets (http://www.insidegadgets.com)
 Created: 1/04/2017
 Last Modified: 26/06/2017
 
 Parts:
 ATtiny841
 ATSHA204A - random numbers and HMAC verification
 Si4432 - 434 MHz radio
 "GPS-12" Reed switch NC
 
 */

#define F_CPU 1000000

#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <string.h>
#include <avr/pgmspace.h>
#include <avr/sleep.h>
#include "si4432.c"
#include "setup.c"
#include "atsha204a.c"
#include "functions.c" // Application specific functions

// ATtiny841 Pin map
//
//                        +-\/-+
//                  VCC  1|o   |14  GND
//                  PB0  2|    |13  PA0 SDA (ATSHA204A)
//        Reed trig PB1  3|    |12  PA1 SCL (ATSHA204A)
//              RST PB3  4|    |11  PA2 ADC for battery
//              LED PB2  5|    |10  PA3 RF22 nIRQ
//   SS (RF22 NSEL) PA7  6|    |9   PA4 SCK (RF22 SCLK)
//  MOSI (RF22 SDI) PA6  7|    |8   PA5 MISO (RF22 SDO)
//                        +----+

int main(void) {
	setup();
	
	uint8_t validateSensor = 0;
	while(1) {
		check_battery_voltage(); // Check if battery is below 3V, if so, disable this sensor
		server_check_in(); // Check in with the server to receive the new system state
		
		// If the sensor isn't disabled, check if the state has changed
		if (sensorDisabled == false && systemStateReceived != systemState) {
			systemState = systemStateReceived;
			
			if (systemState == SYSTEM_ON) {
				PCMSK1 |= (1<<reed_INT); // Apply interrupt
			}
			else {
				PCMSK1 &= ~(1<<reed_INT); // Remove interrupt
			}
		}
		
		// One last check to see if sensor isn't triggered before sleeping
		if (sensorTriggered == false) {
			watchdog_sleep(T8S);
		}
		validateSensor++;
		
		// Sensor triggered, alert server
		if (sensorTriggered == true) {
			PORTB |= (1<<LED);
			
			sensorTriggered = false;
			validateSensor = 0;
			systemState = ALARM_TRIGGERED;
			
			PCMSK1 &= ~(1<<reed_INT); // Remove interrupt
			RH_RF22_setTxPower(RH_RF22_TXPOW_8DBM); // Bump up to 8dBm as this is important
			
			// Keep sending alarm triggered packet until we succeed
			uint8_t attempts = 0;
			while (validate_sensor_to_server() == false && attempts <= 10) {
				watchdog_sleep(T500MS);
				attempts++;
			}
			systemState = systemStateReceived; // Set back
			
			RH_RF22_setTxPower(si443OutputPower); // Set output power back
			PCMSK1 |= (1<<reed_INT); // Apply interrupt
			PORTB &= ~(1<<LED);
			watchdog_sleep(T2S);
		}
		
		// Once every minute or so, validate this sensor to the server
		if (validateSensor >= 8) {
			validateSensor = 0;
			validate_sensor_to_server();
		}
		
		/*PORTB |= (1<<LED);
		_delay_ms(50);
		PORTB &= ~(1<<LED);
		_delay_ms(50);*/
	}
}

ISR(WDT_vect) { }
ISR(ADC_vect) { }
ISR(PCINT0_vect) { }

// Pin change for Reed
ISR(PCINT1_vect) {
	_delay_ms(10);
	
	if (!(PINB & (1<<reedReading))) { // Reed output low
		sensorTriggered = true;
	}
}
