#include <ESP8266WiFi.h>
#include <WiFiClient.h>
#include <ESP8266WebServer.h>
#include <ESP8266mDNS.h>
#include "esp8266-sendemail/sendemail.h"

const char *emailFromName = "Alarm System";
const char *emailFromAddress = "<AlarmSystem@youremail.com>";

const char *ssid = "ssid";
const char *password = "password";
ESP8266WebServer server ( 80 );


#define SYSTEM_ON 1
#define SYSTEM_OFF 2
#define SYSTEM_ARMED 3
#define SYSTEM_TRIGGERED 4
#define SYSTEM_DISABLED 5
uint8_t system_state = SYSTEM_OFF;

#define SIREN_ON 1
#define SIREN_OFF 2
uint8_t siren_state = SIREN_OFF;

#define ALARM_TRIGGERED 231

const int led = 2;
const int rtcInterruptPin = 4;
volatile uint32_t rtcUptime = 1;
String tempCalcTimeStr = "";
uint8_t sensorTypeArmed = 0xFF;

// Sensors structure
#define MAX_SENSORS 30
struct Sensor {
  char name[20];
  char voltage[6];
  char type[10];
  uint8_t zone;
  uint8_t state;
  uint32_t lastseen;
  uint32_t lastvalid;
};
Sensor sensorList[MAX_SENSORS];

#define MAX_LOG 50
char systemLog[MAX_LOG][64];
uint8_t systemLogCount = 0;

char emailAddresses[3][50];


void handleRoot() {
  digitalWrite ( led, 1 );
  
  // Check for alarm system change
  if (server.arg(0) == "Turn On") {
    system_state = SYSTEM_ON;
    Serial.print("A-1");
    delay(100); // Wait a bit for ATmega to update the log
    check_serial_input();
   
    for (uint8_t x = 0; x < 3; x++) {
      if (strlen(emailAddresses[x]) >= 5) {
        SendEmail e("mail.yourisp.com", 25, "", "", 2500, false);
        e.send(emailFromName, emailFromAddress, emailAddresses[x], "Alarm on", "Alarm on");
      }
    }
  }
  else if (server.arg(0) == "Turn Off") {
    Serial.print("A-2");
    delay(100);
    check_serial_input();
    
    // Check if alarm was on, then send an alert about it
    if (system_state == SYSTEM_ON) {
      for (uint8_t x = 0; x < 3; x++) {
        if (strlen(emailAddresses[x]) >= 5) {
          SendEmail e("mail.yourisp.com", 25, "", "", 2500, false);
          e.send(emailFromName, emailFromAddress, emailAddresses[x], "Alarm & Siren off", "Alarm & Siren off");
        }
      }
    }
    
    system_state = SYSTEM_OFF;
    siren_state = SIREN_OFF;
  }
  else if (server.arg(0) == "Arm") {
    system_state = SYSTEM_ARMED;
    Serial.print("A-3");
    delay(100);
    check_serial_input();
  }
  
  char alarm_text[10];
  if (system_state == SYSTEM_OFF) {
    strncpy(alarm_text, "Off", 4);
  }
  else if (system_state == SYSTEM_ON) {
    strncpy(alarm_text, "On", 3);
  }
  else if (system_state == SYSTEM_ARMED) {
    strncpy(alarm_text, "Armed", 6);
  }
   else if (system_state == SYSTEM_TRIGGERED) {
    strncpy(alarm_text, "Triggered", 10);
  }

  char siren_text[10];
  if (siren_state == SIREN_OFF) {
    strncpy(siren_text, "Off", 4);
  }
  else if (siren_state == SIREN_ON) {
    strncpy(siren_text, "On", 3);
  }

  String sensorTypeText = "";
  if (server.arg(0) == "PIR") {
    sensorTypeArmed ^= (1<<0);
    Serial.print("t-");
    Serial.write(sensorTypeArmed);
    delay(100);
  }
  else if (server.arg(0) == "Door") {
    sensorTypeArmed ^= (1<<1);
    Serial.print("t-");
    Serial.write(sensorTypeArmed);
    delay(100);
  }
  else if (server.arg(0) == "Siren") {
    sensorTypeArmed ^= (1<<2);
    Serial.print("t-");
    Serial.write(sensorTypeArmed);
    delay(100);
  }

  
  String temp = "<html>\
<head>\
<title>Alarm System</title>\
</head>\
<link rel='stylesheet' type='text/css' href='style.css'>\
<body>\
<div class='text-css'><p align='center'><b>Alarm is ";

temp += alarm_text;

temp += "</b><br/>Siren is ";

temp += siren_text;

temp += "</p></div>\
<form method='POST' action=''>\
 <p align='center'>";

// PIR
if (sensorTypeArmed & (1<<0)) {
  temp += "<input type='submit' class='typeon-css' name='submit' value='PIR' /> ";
}
else {
  temp += "<input type='submit' class='typeoff-css' name='submit' value='PIR' /> ";
}

// Door
if (sensorTypeArmed & (1<<1)) {
  temp += "<input type='submit' class='typeon-css' name='submit' value='Door' /> ";
}
else {
  temp += "<input type='submit' class='typeoff-css' name='submit' value='Door' /> ";
}

// Siren
if (sensorTypeArmed & (1<<2)) {
  temp += "<input type='submit' class='typeon-css' name='submit' value='Siren' />";
}
else {
  temp += "<input type='submit' class='typeoff-css' name='submit' value='Siren' />";
}

temp += "<br/><br/><input type='submit' class='arm-css' name='submit' value='Arm' />\
  &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;\
  <input type='submit' class='on-css' name='submit' value='Turn On' />\
  &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;\
  <input type='submit' class='off-css' name='submit' value='Turn Off' />\
  </p>\
</form>\
<br/><div class='text-css'><p align='center'><b>Sensors</b></p></div>\
<div align='center'>\
  <table border='1' width='85%' cellspacing='0' cellpadding='0' bordercolorlight='#000000'>\
    <tr>\
      <td align='center' width='21%'><font class='normaltext-css'><b>Sensor</b></font></td>\
      <td align='center' width='9%'><font class='normaltext-css'><b>Zone</b></font></td>\
      <td align='center' width='14%'><font class='normaltext-css'><b>Type</b></font></td>\
      <td align='center'><font class='normaltext-css'><b>Voltage</b></font></td>\
      <td align='center' width='19%'><font class='normaltext-css'><b>Check in</b></font></td>\
      <td align='center' width='19%'><font class='normaltext-css'><b>Valid</b></font></td>\
    </tr>";

  for (uint8_t x = 0; x < MAX_SENSORS; x++) {
    if (sensorList[x].lastseen > 0) {
      temp += "<tr><td align='center' width='24%'><font class='normaltext-css'>";
      temp += (x+1);
      temp += " (";
      temp += sensorList[x].name;
      temp += ")";
      temp += "</font></td><td align='center'><font class='normaltext-css'>";
      temp += sensorList[x].zone;
      temp += "</font></td><td align='center'><font class='normaltext-css'>";
      temp += sensorList[x].type;
      
      temp += "</font></td><td align='center'><font class='normaltext-css'>";
      temp += sensorList[x].voltage;
      temp += "</font></td><td align='center'><font class='normaltext-css'>";

      calculate_last_time(sensorList[x].lastseen);
      temp += tempCalcTimeStr;

      temp += "</font></td><td align='center'><font class='normaltext-css'>";
      calculate_last_time(sensorList[x].lastvalid);
      temp += tempCalcTimeStr;
      
      temp += "(";
      if (sensorList[x].state == SYSTEM_ON) {
        temp += "On";
      }
      else if (sensorList[x].state == SYSTEM_OFF) {
        temp += "Off";
      }
      else if (sensorList[x].state == ALARM_TRIGGERED) {
        temp += "Trig";
      }
      else if (sensorList[x].state == SYSTEM_DISABLED) {
        temp += "Dis";
      }
      
      temp += ")";
      temp += "</font></td></tr>";
    }
  }
  temp += "</table></div>";

  temp += "<br/><br/><div class='text-css'><p align='center'><b>Log</b></p></div>";
  
  //<p align='center'><font class='normaltext-css'>11/10/2016 3:39PM&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; Alarm Armed</font></p>

  temp += "<div align='center'><table border='0' width='66%' cellspacing='0' cellpadding='0' bordercolorlight='#000000'><tr><td><font class='normaltext-css'>";

  uint8_t logCounter = systemLogCount;
  for (uint8_t x = 0; x < MAX_LOG; x++) {
    if (strlen(systemLog[logCounter]) >= 2) {
      /*temp += "c";
      temp += x;
      temp += " l";
      temp += logCounter;
      temp += " ";*/
      temp += systemLog[logCounter];
      temp += "<br/>";
    }
    if (logCounter == 0) {
      logCounter = MAX_LOG-1;
    }
    else {
      logCounter--;
    }
  }
  temp += "</font></td></tr></table><br/><br> Uptime is: ";
  calculate_last_time(1);
  temp += tempCalcTimeStr;
  temp += "</div></body></html>";
  
  server.send ( 200, "text/html", temp );
  digitalWrite ( led, 0 );
  //Serial.println("-done");
}

void handleCss() {
  digitalWrite ( led, 1 );
  //Serial.println("css");
   
  String temp = ".arm-css {\
  display: inline-block;\
  -webkit-box-sizing: content-box;\
  -moz-box-sizing: content-box;\
  box-sizing: content-box;\
  width: 16%;\
  height: 5%;\
  cursor: pointer;\
  padding: 10px 20px;\
  border: 1px solid #000000;\
  -webkit-border-radius: 3px;\
  border-radius: 3px;\
  font: normal 30px/normal 'Trebuchet MS', Helvetica, sans-serif;\
  color: rgba(255,255,255,0.9);\
  -o-text-overflow: clip;\
  text-overflow: clip;\
  background: rgba(232,126,39,0.66);\
  -webkit-box-shadow: 2px 2px 2px 0 rgba(0,0,0,0.2) ;\
  box-shadow: 2px 2px 2px 0 rgba(0,0,0,0.2) ;\
  -webkit-transition: all 300ms cubic-bezier(0.42, 0, 0.58, 1);\
  -moz-transition: all 300ms cubic-bezier(0.42, 0, 0.58, 1);\
  -o-transition: all 300ms cubic-bezier(0.42, 0, 0.58, 1);\
  transition: all 300ms cubic-bezier(0.42, 0, 0.58, 1);\
}\
.on-css {\
  display: inline-block;\
  -webkit-box-sizing: content-box;\
  -moz-box-sizing: content-box;\
  box-sizing: content-box;\
  width: 16%;\
  height: 5%;\
  cursor: pointer;\
  padding: 10px 20px;\
  border: 1px solid #000000;\
  -webkit-border-radius: 3px;\
  border-radius: 3px;\
  font: normal 30px/normal 'Trebuchet MS', Helvetica, sans-serif;\
  color: rgba(255,255,255,0.9);\
  -o-text-overflow: clip;\
  text-overflow: clip;\
  background: rgba(216,2,2,0.9);\
  -webkit-box-shadow: 2px 2px 2px 0 rgba(0,0,0,0.2) ;\
  box-shadow: 2px 2px 2px 0 rgba(0,0,0,0.2) ;\
  -webkit-transition: all 300ms cubic-bezier(0.42, 0, 0.58, 1);\
  -moz-transition: all 300ms cubic-bezier(0.42, 0, 0.58, 1);\
  -o-transition: all 300ms cubic-bezier(0.42, 0, 0.58, 1);\
  transition: all 300ms cubic-bezier(0.42, 0, 0.58, 1);\
}\
.off-css {\
  display: inline-block;\
  -webkit-box-sizing: content-box;\
  -moz-box-sizing: content-box;\
  box-sizing: content-box;\
  width: 16%;\
  height: 5%;\
  cursor: pointer;\
  padding: 10px 20px;\
  border: 1px solid #000000;\
  -webkit-border-radius: 3px;\
  border-radius: 3px;\
  font: normal 30px/normal 'Trebuchet MS', Helvetica, sans-serif;\
  color: rgba(255,255,255,0.9);\
  -o-text-overflow: clip;\
  text-overflow: clip;\
  background: #0199d9;\
  -webkit-box-shadow: 2px 2px 2px 0 rgba(0,0,0,0.2) ;\
  box-shadow: 2px 2px 2px 0 rgba(0,0,0,0.2) ;\
  text-shadow: -1px -1px 0 rgba(15,73,168,0.66) ;\
  -webkit-transition: all 300ms cubic-bezier(0.42, 0, 0.58, 1);\
  -moz-transition: all 300ms cubic-bezier(0.42, 0, 0.58, 1);\
  -o-transition: all 300ms cubic-bezier(0.42, 0, 0.58, 1);\
  transition: all 300ms cubic-bezier(0.42, 0, 0.58, 1);\
}\
.typeon-css {\
  display: inline-block;\
  -webkit-box-sizing: content-box;\
  -moz-box-sizing: content-box;\
  box-sizing: content-box;\
  width: 7%;\
  height: 2%;\
  cursor: pointer;\
  padding: 10px 20px;\
  border: 1px solid #000000;\
  -webkit-border-radius: 3px;\
  border-radius: 3px;\
  font: normal 20px/normal 'Trebuchet MS', Helvetica, sans-serif;\
  color: rgba(255,255,255,0.9);\
  -o-text-overflow: clip;\
  text-overflow: clip;\
  background: rgba(0,170,2,0.9);\
  -webkit-box-shadow: 2px 2px 2px 0 rgba(0,0,0,0.2) ;\
  box-shadow: 2px 2px 2px 0 rgba(0,0,0,0.2) ;\
  -webkit-transition: all 300ms cubic-bezier(0.42, 0, 0.58, 1);\
  -moz-transition: all 300ms cubic-bezier(0.42, 0, 0.58, 1);\
  -o-transition: all 300ms cubic-bezier(0.42, 0, 0.58, 1);\
  transition: all 300ms cubic-bezier(0.42, 0, 0.58, 1);\
}\
.typeoff-css {\
  display: inline-block;\
  -webkit-box-sizing: content-box;\
  -moz-box-sizing: content-box;\
  box-sizing: content-box;\
  width: 7%;\
  height: 2%;\
  cursor: pointer;\
  padding: 10px 20px;\
  border: 1px solid #000000;\
  -webkit-border-radius: 3px;\
  border-radius: 3px;\
  font: normal 20px/normal 'Trebuchet MS', Helvetica, sans-serif;\
  color: rgba(255,255,255,0.9);\
  -o-text-overflow: clip;\
  text-overflow: clip;\
  background: rgba(216,2,2,0.9);\
  -webkit-box-shadow: 2px 2px 2px 0 rgba(0,0,0,0.2) ;\
  box-shadow: 2px 2px 2px 0 rgba(0,0,0,0.2) ;\
  -webkit-transition: all 300ms cubic-bezier(0.42, 0, 0.58, 1);\
  -moz-transition: all 300ms cubic-bezier(0.42, 0, 0.58, 1);\
  -o-transition: all 300ms cubic-bezier(0.42, 0, 0.58, 1);\
  transition: all 300ms cubic-bezier(0.42, 0, 0.58, 1);\
}\
.text-css {\
  -webkit-box-sizing: content-box;\
  -moz-box-sizing: content-box;\
  box-sizing: content-box;\
  border: none;\
  font: normal 40px/1 'Trebuchet MS', Helvetica, sans-serif;\
  color: black;\
  -o-text-overflow: ellipsis;\
  text-overflow: ellipsis;\
}\
.normaltext-css {\
  -webkit-box-sizing: content-box;\
  -moz-box-sizing: content-box;\
  box-sizing: content-box;\
  border: none;\
  font: normal 30px/1 'Trebuchet MS', Helvetica, sans-serif;\
  color: black;\
  -o-text-overflow: ellipsis;\
  text-overflow: ellipsis;\
}";

  server.send ( 200, "text/html", temp );
  digitalWrite ( led, 0 );
  //Serial.println("-done");
}

void handleAdmin() {
  digitalWrite ( led, 1 );

  String temp = "<html>\
<head>\
<title>Alarm System - Admin</title>\
</head>\
<link rel='stylesheet' type='text/css' href='style.css'>\
<body>";
  
  if (server.hasArg("day")) {
    temp += "Day = " + server.arg("day") + "<br/>";
    temp += "Month = " + server.arg("month") + "<br/>";
    temp += "Year = " + server.arg("year") + "<br/>";
    temp += "Hour = " + server.arg("hour") + "<br/>";
    temp += "Minute = " + server.arg("minute") + "<br/>";
    temp += "Second = " + server.arg("second") + "<br/>";
    temp += "AM/PM = " + server.arg("ampm") + "<br/><br/>";

    Serial.print("D-");
    Serial.print(server.arg("year"));
    Serial.print("-");
    Serial.print(server.arg("month"));
    Serial.print("-");
    Serial.print(server.arg("day"));
    Serial.print("-");
    Serial.print(server.arg("hour"));
    Serial.print("-");
    Serial.print(server.arg("minute"));
    Serial.print("-");
    Serial.print(server.arg("second"));
    Serial.print("-");
    if (server.arg("ampm") == "AM") {
      Serial.print("0");
    }
    else {
      Serial.print("1");
    }
    Serial.print("-");
  }
  else if (server.hasArg("sensorname")) {
    temp += "Sensor = " + server.arg("sensor") + "<br/>";
    temp += "Name = " + server.arg("sensorname") + "<br/><br/>";
    
    Serial.print("N-");
    Serial.print(server.arg("sensor"));
    Serial.print("-");
    Serial.print(server.arg("sensorname"));
    Serial.print("-");
  }
  else if (server.hasArg("type")) {
    temp += "Sensor = " + server.arg("sensor") + "<br/>";
    temp += "Type = " + server.arg("type") + "<br/>";
    temp += "Zone = " + server.arg("zone") + "<br/><br/>";

    Serial.print("U-");
    Serial.print(server.arg("sensor"));
    Serial.print("-");
    Serial.print(server.arg("type"));
    Serial.print("-");
    Serial.print(server.arg("zone"));
    Serial.print("-");
  }

  else if (server.hasArg("emailslot")) {
    temp += "Email slot = " + server.arg("emailslot") + "<br/>";
    temp += "Email address = " + server.arg("emailaddress") + "<br/><br/>";

    Serial.print("E-");
    Serial.print(server.arg("emailslot"));
    Serial.print("-");
    Serial.print(server.arg("emailaddress"));
    Serial.print("-");

    // Update local email addresses
    uint8_t emailSlot = server.arg("emailslot").toInt() - 1;
    strncpy(emailAddresses[emailSlot], server.arg("emailaddress").c_str(), 50);
  }
  
  temp += "<form method='POST' action=''>\
  <p align='center'>Date <select size='1' name='day'>\
  <option>1</option><option>2</option><option>3</option><option>4</option><option>5</option><option>6</option><option>7</option><option>8</option><option>9</option><option>10</option>\
  <option>11</option><option>12</option><option>13</option><option>14</option><option>15</option><option>16</option><option>17</option><option>18</option><option>19</option><option>20</option>\
  <option>21</option><option>22</option><option>23</option><option>24</option><option>25</option><option>26</option><option>27</option><option>28</option><option>29</option><option>30</option>\
  <option>31</option>\
  </select> Month <select size='1' name='month'>\
  <option>1</option><option>2</option><option>3</option><option>4</option><option>5</option><option>6</option><option>7</option><option>8</option><option>9</option><option>10</option>\
  <option>11</option><option>12</option></select> Year <select size='1' name='year'>\
  <option>17</option><option>18</option><option>19</option><option>20</option><option>21</option>\
  </select><br>\
  Hour <select size='1' name='hour'>\
  <option>1</option><option>2</option><option>3</option><option>4</option><option>5</option><option>6</option><option>7</option><option>8</option><option>9</option><option>10</option>\
  <option>11</option><option>12</option>\
  </select> Minute <select size='1' name='minute'>\
   <option>0</option><option>1</option><option>2</option><option>3</option><option>4</option><option>5</option><option>6</option><option>7</option><option>8</option><option>9</option><option>10</option>\
  <option>11</option><option>12</option><option>13</option><option>14</option><option>15</option><option>16</option><option>17</option><option>18</option><option>19</option><option>20</option>\
  <option>21</option><option>22</option><option>23</option><option>24</option><option>25</option><option>26</option><option>27</option><option>28</option><option>29</option><option>30</option>\
  <option>31</option><option>32</option><option>33</option><option>34</option><option>35</option><option>36</option><option>37</option><option>38</option><option>39</option><option>40</option>\
  <option>41</option><option>42</option><option>43</option><option>44</option><option>45</option><option>46</option><option>47</option><option>48</option><option>49</option><option>50</option>\
  <option>51</option><option>52</option><option>53</option><option>54</option><option>55</option><option>56</option><option>57</option><option>58</option><option>59</option>\
  </select>\
   Second <select size='1' name='second'>\
   <option>0</option><option>1</option><option>2</option><option>3</option><option>4</option><option>5</option><option>6</option><option>7</option><option>8</option><option>9</option><option>10</option>\
  <option>11</option><option>12</option><option>13</option><option>14</option><option>15</option><option>16</option><option>17</option><option>18</option><option>19</option><option>20</option>\
  <option>21</option><option>22</option><option>23</option><option>24</option><option>25</option><option>26</option><option>27</option><option>28</option><option>29</option><option>30</option>\
  <option>31</option><option>32</option><option>33</option><option>34</option><option>35</option><option>36</option><option>37</option><option>38</option><option>39</option><option>40</option>\
  <option>41</option><option>42</option><option>43</option><option>44</option><option>45</option><option>46</option><option>47</option><option>48</option><option>49</option><option>50</option>\
  <option>51</option><option>52</option><option>53</option><option>54</option><option>55</option><option>56</option><option>57</option><option>58</option><option>59</option>\
  </select>\
  <select size='1' name='ampm'>\
  <option>AM</option><option>PM</option>\
  </select></p>\
  <p align='center'><input type='submit' value='Submit' name='daytime'></p>\
  </form>";

  temp += "<form method='POST' action=''>\
  <p align='center'>Sensor <select size='1' name='sensor'>\
  <option>1</option><option>2</option><option>3</option><option>4</option><option>5</option><option>6</option><option>7</option><option>8</option><option>9</option><option>10</option>\
  <option>11</option><option>12</option><option>13</option><option>14</option><option>15</option><option>16</option><option>17</option><option>18</option><option>19</option><option>20</option>\
  <option>21</option><option>22</option><option>23</option><option>24</option><option>25</option><option>26</option><option>27</option><option>28</option><option>29</option><option>30</option>\
  <option>31</option><option>32</option><option>33</option><option>34</option><option>35</option><option>36</option><option>37</option><option>38</option><option>39</option><option>40</option>\
  <option>41</option><option>42</option><option>43</option><option>44</option><option>45</option><option>46</option><option>47</option><option>48</option><option>49</option><option>50</option>\
  </select><br/>\
  Name <input type='text' name='sensorname' size='19'></p>\
  <p align='center'><input type='submit' value='Submit' name='sensorupdate'></p>\
  </form>";

  temp += "<form method='POST' action=''>\
  <p align='center'>Sensor <select size='1' name='sensor'>\
  <option>1</option><option>2</option><option>3</option><option>4</option><option>5</option><option>6</option><option>7</option><option>8</option><option>9</option><option>10</option>\
  <option>11</option><option>12</option><option>13</option><option>14</option><option>15</option><option>16</option><option>17</option><option>18</option><option>19</option><option>20</option>\
  <option>21</option><option>22</option><option>23</option><option>24</option><option>25</option><option>26</option><option>27</option><option>28</option><option>29</option><option>30</option>\
  <option>31</option><option>32</option><option>33</option><option>34</option><option>35</option><option>36</option><option>37</option><option>38</option><option>39</option><option>40</option>\
  <option>41</option><option>42</option><option>43</option><option>44</option><option>45</option><option>46</option><option>47</option><option>48</option><option>49</option><option>50</option>\
  </select><br/>\
  Type <select size='1' name='type'>\
  <option value='1'>PIR</option><option value='2'>Door</option><option value='3'>Siren</option>\
  </select><br/>\
  Zone <select size='1' name='zone'>\
  <option>1</option><option>2</option><option>3</option><option>4</option><option>5</option><option>6</option><option>7</option><option>8</option><option>9</option><option>10</option>\
  <option>11</option><option>12</option><option>13</option><option>14</option><option>15</option><option>16</option><option>17</option><option>18</option><option>19</option><option>20</option>\
  <option>21</option><option>22</option><option>23</option><option>24</option><option>25</option><option>26</option><option>27</option><option>28</option><option>29</option><option>30</option>\
  <option>31</option><option>32</option><option>33</option><option>34</option><option>35</option><option>36</option><option>37</option><option>38</option><option>39</option><option>40</option>\
  <option>41</option><option>42</option><option>43</option><option>44</option><option>45</option><option>46</option><option>47</option><option>48</option><option>49</option><option>50</option>\
  </select></p>\
  <p align='center'><input type='submit' value='Submit' name='sensortypeupdate'></p>\
  </form>";

  temp += "<form method='POST' action=''>\
  <p align='center'>Email Slot <select size='1' name='emailslot'>\
  <option>1</option><option>2</option><option>3</option></select><br/>\
  Email address <input type='text' name='emailaddress' size='19'></p>\
  <p align='center'><input type='submit' value='Submit' name='emailupdate'></p>\
  </form>";

  server.send ( 200, "text/html", temp );
  digitalWrite ( led, 0 );
}

void handleNotFound() {
  digitalWrite ( led, 1 );
  
  String message = "File Not Found\n\n";
  message += "URI: ";
  message += server.uri();
  message += "\nMethod: ";
  message += ( server.method() == HTTP_GET ) ? "GET" : "POST";
  message += "\nArguments: ";
  message += server.args();
  message += "\n";

  for ( uint8_t i = 0; i < server.args(); i++ ) {
    message += " " + server.argName ( i ) + ": " + server.arg ( i ) + "\n";
  }

  server.send ( 404, "text/plain", message );
  digitalWrite ( led, 0 );
}

// Check if serial input is available
void check_serial_input(void) {
  if (Serial.available() > 0) {
    delay(10);

    // Decode input
    char readInput[64];
    int readCount = 0;
    while (Serial.available() > 0 && readCount < 63) {
      char c = Serial.read();
      readInput[readCount] = c;
      readCount++;
    }
    readInput[readCount] = '\0';
    
    // Update sensor, (i.e S-<sensor>-<zone>-<type>-<name>-<voltage>-) e.g. (i.e S-1-1-PIR-Kitchen-3.5V-)
    if (readInput[0] == 'S') {
      //Serial.println ("Updating Sensor");
      const char delim[2] = "-";
      uint8_t tokcount = 0;
    
      uint8_t sensorNo = 0;
      char timeSeen[3];

      char *token = strtok(readInput, delim);
      while(token != NULL) {
        //Serial.println(token);
        if (tokcount == 1) { // Sensor number
          if (atoi(token) <= 0 || atoi(token) >= MAX_SENSORS) { // Out of bounds
            break;
          }
          
          sensorNo = atoi(token) - 1;
        }
        else if (tokcount == 2) { // Zone
          sensorList[sensorNo].zone = atoi(token);
        }
        else if (tokcount == 3) { // Type
          strncpy(sensorList[sensorNo].type, token, 9);
          sensorList[sensorNo].type[9] = 0;
        }
        else if (tokcount == 4) { // Sensor name
          strncpy(sensorList[sensorNo].name, token, 18);
          sensorList[sensorNo].name[19] = 0;
        }        
        else if (tokcount == 5) { // Voltage
          strncpy(sensorList[sensorNo].voltage, token, 5);
          sensorList[sensorNo].voltage[5] = 0;

          // Update last seen time
          sensorList[sensorNo].lastseen = rtcUptime;
        }
              
        tokcount++;
        token = strtok(NULL, delim);
      }
    }

    // Update sensor valid
    else if (readInput[0] == 'V') {
      const char delim[2] = "-";
      uint8_t tokcount = 0;
      uint8_t sensorNo = 0;
      
      char *token = strtok(readInput, delim);
      while(token != NULL) {
        if (tokcount == 1) { // Sensor number
          if (atoi(token) <= 0 || atoi(token) >= MAX_SENSORS) { // Out of bounds
            break;
          }
          sensorNo = atoi(token) - 1;
        }
        else if (tokcount == 2) { // Sensor state
          sensorList[sensorNo].state = atoi(token);
          
          // Update last valid time
          sensorList[sensorNo].lastvalid = rtcUptime;
        }

        tokcount++;
        token = strtok(NULL, delim);
      }
    }

    // Update log, e.g L-Text
    else if (readInput[0] == 'L' && readInput[1] == '-') {
      //Serial.println ("Updating Log");
      if (systemLogCount == MAX_LOG-1) {
        systemLogCount = 0;
      }
      else {
        systemLogCount++;
      }
      
      strncpy(systemLog[systemLogCount], &readInput[2] , readCount+1);
    }

    // Alarm triggered, siren on/off
    else if (readInput[0] == 'T' && readInput[1] == '-') {
      if (readInput[2] == '1') {
        siren_state = SIREN_ON;

        for (uint8_t x = 0; x < 3; x++) {
          if (strlen(emailAddresses[x]) >= 5) {
            SendEmail e("mail.yourisp.com", 25, "", "", 2500, false);
            e.send(emailFromName, emailFromAddress, emailAddresses[x], "Alarm triggered", "Siren on");
          }
        }
      }
      else if (readInput[2] == '2') {
        siren_state = SIREN_OFF;
        
        for (uint8_t x = 0; x < 3; x++) {
          if (strlen(emailAddresses[x]) >= 5) {
            SendEmail e("mail.yourisp.com", 25, "", "", 2500, false);
            e.send(emailFromName, emailFromAddress, emailAddresses[x], "Alarm siren off ", "Siren off");
          }
        }
      }
    }

    // Update email addresses, e.g E-1-me@example.com-
    else if (readInput[0] == 'E' && readInput[1] == '-') {
      const char delim[2] = "-";
      uint8_t tokcount = 0;
      uint8_t emailSlot = 0;
      
      char *token = strtok(readInput, delim);
      while(token != NULL) {
        if (tokcount == 1) { // Email slot
          if (atoi(token) <= 0 || atoi(token) >= 3) { // Out of bounds
            break;
          } 
          emailSlot = atoi(token) - 1;
        }
        
        else if (tokcount == 2) { // Type
          strncpy(emailAddresses[emailSlot], token, 50);
        }
              
        tokcount++;
        token = strtok(NULL, delim);
      }
    }
  }
}

void calculate_last_time(uint32_t sensorTime) {  
  uint32_t calcTime = rtcUptime - sensorTime;
  uint8_t calcDays = 0;
  uint8_t calcHours = 0;
  uint8_t calcMinutes = 0;
  uint8_t calcSeconds = 0;
  
  tempCalcTimeStr = "";

  if (sensorTime >= 1) {
    if (calcTime / 86400 >= 1) {
      calcDays = calcTime / 86400;
      calcTime = calcTime - (86400 * calcDays);
      tempCalcTimeStr += calcDays;
      tempCalcTimeStr += "d, ";
    }
    if (calcTime / 3600 >= 1) {
      calcHours = calcTime / 3600;
      calcTime = calcTime - (3600 * calcHours);
      tempCalcTimeStr += calcHours;
      tempCalcTimeStr += "h, ";
    }
    if (calcTime / 60 >= 1) {
      calcMinutes = calcTime / 60;
      calcTime = calcTime - (60 * calcMinutes);
      tempCalcTimeStr += calcMinutes;
      tempCalcTimeStr += "m, ";
    }
    calcSeconds = calcTime;
    
    tempCalcTimeStr += calcSeconds;
    tempCalcTimeStr += "s";
  }
}

void ICACHE_RAM_ATTR uptimeUpdate(void) {
  rtcUptime++;
}

void setup ( void ) {
  pinMode ( led, OUTPUT );
  digitalWrite ( led, 0 );
  Serial.begin ( 115200 );
  WiFi.mode(WIFI_STA);
  WiFi.begin ( ssid, password );
  
  // Clear Log text
  for (uint8_t x = 0; x < MAX_LOG; x++) {
    systemLog[x][0] = 0;
  }
  // Clear email addresses
  for (uint8_t x = 0; x < 3; x++) {
    emailAddresses[x][0] = 0;
  }
  
  // Wait for connection
  while ( WiFi.status() != WL_CONNECTED ) {
    delay ( 500 );
    //Serial.print ( "." );
  }

  //Serial.println ( "" );
  //Serial.print ( "Connected to " );
  //Serial.println ( ssid );
  //Serial.print ( "IP address: " );
 // Serial.println ( WiFi.localIP() );

  //if ( MDNS.begin ( "esp8266" ) ) {
    //Serial.println ( "MDNS responder started" );
  //}

  server.on ( "/", handleRoot );
  server.on ( "/admin", handleAdmin );
  server.on ( "/style.css", handleCss );
  server.onNotFound ( handleNotFound );
  server.begin();
  //Serial.println ( "HTTP server started" );

  Serial.print('W'); // Wake up ATmega
  delay(5);
  
  // Read log (received newest to oldest)
  uint8_t inLogRead = true;
  uint8_t delaycount = 0;
  systemLogCount = (MAX_LOG - 1);
  while (inLogRead == true) {
    if (Serial.available() > 0) {
      delaycount = 0;
      delay(50);
  
      // Decode input
      char readInput[64];
      int readCount = 0;
      while (Serial.available() > 0 && readCount < 63) {
        char c = Serial.read();
        readInput[readCount] = c;
        readCount++;
      }
      readInput[readCount] = '\0';
      
      for (uint8_t c = 0; c < readCount; c++) {
        systemLog[systemLogCount][c] = readInput[c];
      }   
      systemLogCount--;
    }
    
    // Timeout after 250ms of no data received
    delay(5);
    delaycount++;
    if (delaycount >= 50) {
      inLogRead = false;
    }
  }

  systemLogCount = MAX_LOG-1; // Reset count to newest

  attachInterrupt(digitalPinToInterrupt(rtcInterruptPin), uptimeUpdate, RISING);
}

void loop ( void ) {
  server.handleClient();
  check_serial_input();
}



