/*
 Alarm System v3.0 PIR
 Version: Test
 Author: Alex from insideGadgets (http://www.insidegadgets.com)
 Created: 1/04/2017
 Last Modified: 26/06/2017
 
 */

#ifndef NULL
#define NULL ((void *)0)
#endif

#define LOW 0
#define HIGH 1
#define false 0
#define true 1

// Watchdog timeouts to values
#define T16MS 0
#define T32MS 1
#define T64MS 2
#define T128MS 3
#define T250MS 4
#define T500MS 5
#define T1S 6
#define T2S 7
#define T4S 8
#define T8S 9

// Alarm states/vars
#define SYSTEM_ON 1
#define SYSTEM_OFF 2
#define SYSTEM_ARMED 3
#define SYSTEM_TEST 4
#define SYSTEM_DISABLED 5
#define SIREN_ON 1
#define SIREN_OFF 2

#define ALARM_TRIGGERED 231


#define PCINT10 2
#define PCINT3 3

#define pirReading PB2
#define pir_INT PCINT10
#define pirPmosfet PB1
#define LED PB0 // Unused

uint8_t systemState = SYSTEM_OFF;
uint8_t systemStateReceived = SYSTEM_OFF;
uint8_t sensorDisabled = false;
volatile uint8_t sensorTriggered = false;

int8_t checkInCounter = 0;
uint8_t si443OutputPower = RH_RF22_TXPOW_1DBM;

// ADC/Battery settings
int batteryVoltage = 0;
#define lowVoltageLifeBat 489 // ~3V
#define REF_1_1V (1<<REFS0)

#define batteryAdcPin 2 // ADC2 / PA2
int8_t adcCalibration = 0;

// I2C settings
#define TWI_SCL_PIN PA1
#define TWI_SDA_PIN PA0

#define I2C_READ 1
#define I2C_WRITE 0
#define NOREPEATSTART 0
#define REPEATSTART 1
uint8_t I2C_DELAY_CYCLES = 2;

//ATSHA204A
#define ATSHA204A_ADDR 0xC8

// Global vars
uint8_t dataOut[40];
uint8_t dataIn[40];
uint8_t dataBuffer[40];
uint8_t crc[2];
uint8_t randomNumber[32];

extern uint8_t RH_RF22_init(void);


// Sleep
void system_sleep(void) {
	set_sleep_mode(SLEEP_MODE_PWR_DOWN); // Set sleep mode
	sleep_mode(); // System sleeps here
}
// Setup watchdog timed sequence
void setup_watchdog(uint8_t wdDelay) {
	if (wdDelay > 9) { 
		wdDelay = 9;
	}
	
	uint8_t bb = wdDelay & 7;
	if (wdDelay > 7) { 
		bb |= (1<<5);
	}
	
	MCUSR &= ~(1<<WDRF);
	// Start timed sequence
	WDTCSR |= (1<<WDE);
	// Set new watchdog timeout value
	WDTCSR = bb;
	WDTCSR |= _BV(WDIE);
}

// Turn off the Watchdog
void turn_off_watchdog(void) {
	cli(); // Turn interrupts off
	MCUSR &= ~(1<<WDRF);
	CCP = 0xD8;
	WDTCSR = 0;
	sei(); // Turn interrupts back on
}

// Watchdog sleep function - setup, sleep and turn off
void watchdog_sleep(uint8_t timeout) { 
	setup_watchdog(timeout);
	system_sleep();
	turn_off_watchdog();
}

// ADC Read
int adc_read(uint8_t pin) {
	uint8_t low, high;
	
	ADMUXA = (pin & 0x3f);
	ADMUXB = REF_1_1V; // 1.1V voltage reference
	
	set_sleep_mode(SLEEP_MODE_ADC); // Set sleep mode ADC
	sleep_mode(); // System sleeps here
	
	// Read ADCL first
	low = ADCL;
	high = ADCH;
	
	// Combine the two bytes
	return (high << 8) | low;
}

// Read a byte from I2C and send Ack if more reads follow else Nak to terminate read
uint8_t soft_i2c_master_read(uint8_t last) {
	uint8_t b = 0;
	for (uint8_t i = 0; i < 8; i++) {
		// Don't change this loop unless you verify the change with a scope
		b <<= 1;
		_delay_loop_1(I2C_DELAY_CYCLES);
		DDRA &= ~(1<<TWI_SCL_PIN);
		_delay_loop_1(I2C_DELAY_CYCLES);
		if (bit_is_set(PINA, TWI_SDA_PIN)) b |= 1;
		_delay_loop_1(I2C_DELAY_CYCLES);
		DDRA |= (1<<TWI_SCL_PIN);
	}
	_delay_loop_1(I2C_DELAY_CYCLES);
	
	// Send Ack or Nak
	if (last) {
		DDRA &= ~(1<<TWI_SDA_PIN);
	}
	else {
		DDRA |= (1<<TWI_SDA_PIN);
	}

	DDRA &= ~(1<<TWI_SCL_PIN);
	_delay_loop_1(I2C_DELAY_CYCLES);
	DDRA |= (1<<TWI_SCL_PIN);
	DDRA &= ~(1<<TWI_SDA_PIN);
	
	return b;
}

// Write a byte to I2C
uint8_t soft_i2c_master_write(uint8_t data) {
	uint8_t rtn = 0;
	
	// Write byte
	for (uint8_t m = 0x80; m != 0; m >>= 1) {
		// Don't change this loop unless you verify the change with a scope
		if (m & data) { 
			DDRA &= ~(1<<TWI_SDA_PIN);
		}
		else { 
			DDRA |= (1<<TWI_SDA_PIN);
		}
		
		_delay_loop_1(I2C_DELAY_CYCLES);
		DDRA &= ~(1<<TWI_SCL_PIN);
		_delay_loop_1(I2C_DELAY_CYCLES);
		DDRA |= (1<<TWI_SCL_PIN);
		_delay_loop_1(I2C_DELAY_CYCLES);
	}
	_delay_loop_1(I2C_DELAY_CYCLES);
	
	DDRA &= ~(1<<TWI_SDA_PIN);
	DDRA &= ~(1<<TWI_SCL_PIN);
	_delay_loop_1(I2C_DELAY_CYCLES);
	
	// get Ack or Nak
	rtn = bit_is_set(PINA, TWI_SDA_PIN);
	
	DDRA |= (1<<TWI_SCL_PIN);
	DDRA &= ~(1<<TWI_SDA_PIN);
	_delay_loop_1(I2C_DELAY_CYCLES);
	
	return rtn == 0;
}

// Issue a start condition
uint8_t soft_i2c_master_start(uint8_t addressRW) {
	DDRA |= (1<<TWI_SDA_PIN);
	_delay_loop_1(I2C_DELAY_CYCLES);
	DDRA |= (1<<TWI_SCL_PIN);
	_delay_loop_1(I2C_DELAY_CYCLES);
	
	return soft_i2c_master_write(addressRW);
}

// Issue a restart condition
uint8_t soft_i2c_master_restart(uint8_t addressRW) {
	DDRA &= ~(1<<TWI_SDA_PIN);
	DDRA &= ~(1<<TWI_SCL_PIN);
	_delay_loop_1(I2C_DELAY_CYCLES);
	
	return soft_i2c_master_start(addressRW);
}

// Issue a stop condition
void soft_i2c_master_stop(void) {
	DDRA |= (1<<TWI_SDA_PIN);
	_delay_loop_1(I2C_DELAY_CYCLES);
	
	DDRA &= ~(1<<TWI_SCL_PIN);
	_delay_loop_1(I2C_DELAY_CYCLES);
	
	DDRA &= ~(1<<TWI_SDA_PIN);
	_delay_loop_1(I2C_DELAY_CYCLES);
}

// Issue the device start and the 8bit address to write
uint8_t soft_i2c_master_start_with_8bit_address(uint8_t deviceAddr, uint16_t addressRW, uint8_t repeatedStart) {
	if (!soft_i2c_master_start((deviceAddr<<1) | I2C_WRITE)) return false;
	if (!soft_i2c_master_write(addressRW)) return false;
	if (repeatedStart) { if (!soft_i2c_master_restart((deviceAddr<<1) | I2C_READ)) return false; }
	
	return true;
}

// SPI transfer 1 byte and return the result
uint8_t spi_transfer(uint8_t data) {
	SPDR = data;
	
	// Wait for transmission complete
	while (!(SPSR & (1<<SPIF)));
	
	return SPDR;
}

// Initialise the SPI / SS
void spi_init(void) {
	DDRA |= (1<<PA4); // SPI CLK
	DDRA &= ~(1<<PA5); // SPI DI
	PORTA |= (1<<PA5); // SPI DI
	DDRA |= (1<<PA6); // SPI DO
	
	DDRA |= (1<<SS); // SPI SS
	si4432_SS_hi;
	
	// Enable SPI, Master, set clock rate fck/16
	SPCR = (1<<SPE)|(1<<MSTR)|(1<<SPR0);
}

// Setup
void setup(void) {
	// Turn off watchdog
	MCUSR &= ~(1<<WDRF);
	CCP = 0xD8;
	WDTCSR = 0;
	
	_delay_ms(500); // Wait a little bit after power on
	
	// Set outputs
	DDRB |= (1<<pirPmosfet);
	PORTB |= (1<<pirPmosfet); // Turn off PIR (P mosfet)
	
	// Turn on pull-up resistors on other ports to save power
	PORTB |= (1<<PB0);
	
	// Initialise SPI
	spi_init();
	
	// Initialise RF22
	if (RH_RF22_init()) {
	}
	
	// Initialse I2C for ATSHA204A
	DDRA &= ~(1<<TWI_SCL_PIN);
	PORTA &= ~(1<<TWI_SCL_PIN);
	DDRA &= ~(1<<TWI_SDA_PIN);
	PORTA &= ~(1<<TWI_SDA_PIN);
	
	// Set ADC prescale factor to 8
	// 1 MHz / 8 = 125 KHz, inside the desired 50-200 KHz range.
	// Turn on ADC interrupt too
	ADCSRA |= (1<<ADPS1) | (1<<ADPS0) | (1<<ADIE);
	
	// Setup the pin change which will be activated on Si4432 and PIR later on
	GIMSK |= (1<<PCIE0) | (1<<PCIE1);
	
	sei(); // Turn on interrupts
}
