/*
 Alarm System v3.0 Server
 Version: Test
 Author: Alex from insideGadgets (http://www.insidegadgets.com)
 Created: 1/04/2017
 Last Modified: 1/06/2017
 
*/

// Clear data in
void clear_data_in(void) {
	for (uint8_t x = 0; x < 40; x++) {
		dataIn[x] = 0;
	}
}

// Wait for a valid packet
uint8_t wait_for_valid_packet(uint8_t timeoutEnabled) {
	RH_RF22_resetFifos();
	RH_RF22_setModeRx();
	
	// Wait for a packet that matches our address
	uint8_t readReg = 0;
	uint8_t timeoutCounter = 0;
	while (!(readReg & ipkvalid)) {
		readReg = spi_read_reg(RH_RF22_REG_03_INTERRUPT_STATUS1);
		_delay_ms(1);
		PORTB |= (1<<LED);
		
		timeoutCounter++;
		if (timeoutCounter >= 250) { // 250ms timeout
			RH_RF22_setModeRx();
			RH_RF22_resetFifos();
			if (timeoutEnabled == true) {
				return false;
			}
		}
		if (receivedUsart == true) { // Alarm state changed
			return false;
		}
		if (sirenStateCounter >= 54) { // Siren on for 5 minutes
			return false;
		}
	}
	
	clear_data_in();
	
	// Read from address and tpye
	RH_RF22_fromAddress = spi_read_reg(RH_RF22_REG_48_RECEIVED_HEADER2);
	RH_RF22_rxRequestType = spi_read_reg(RH_RF22_REG_49_RECEIVED_HEADER1);
	RH_RF22_rxInfo = spi_read_reg(RH_RF22_REG_4A_RECEIVED_HEADER0);
	
	// Read the packet
	RH_RF22_packetLength = spi_read_reg(RH_RF22_REG_4B_RECEIVED_PACKET_LENGTH); 
	
	// Check for packet length
	if (RH_RF22_packetLength <= 40) {
		spiBurstRead(RH_RF22_REG_7F_FIFO_ACCESS, dataIn, RH_RF22_packetLength);
	}
	
	return true;
}

// Transmit a packet with the request type
void send_packet(uint8_t requestType, uint8_t *packetBuffer, uint8_t packetLength) {
	RH_RF22_resetFifos();
	RH_RF22_clearInterrupts();
	RH_RF22_txHeaderTo = RH_RF22_fromAddress;
	RH_RF22_txHeaderRequestType = requestType;
	RH_RF22_txHeaderSystemState = systemStateToSend;
	RH_RF22_send(packetBuffer, packetLength);
	RH_RF22_wait_for_transmit();
}

// Write event to i2c EEPROM and ESP8266, update log counter on internal EEPROM
void update_log_and_send(void) {
	uint16_t eepromLocation = 0;
	
	if (logCurrentNo != blankEeprombyte) {
		logCurrentNo++;
		if (logCurrentNo >= MAX_LOG) {
			logCurrentNo = 0;
		}
		
		eepromLocation = (logCurrentNo * 64);
	}
	else {
		logCurrentNo = 0;
	}
	
	soft_i2c_eeprom_page_write_64_bytes(EEPROM_ADDR, eepromLocation);
	eeprom_write_byte((uint8_t*) logCurrentNoLocation, logCurrentNo);

	// Update ESP8266
	_delay_ms(25); // Must wait a little bit
	USART_Transmit('L');
	USART_Transmit('-');
	for (uint8_t x = 0; x < strlen(eepromBuffer); x++) {
		USART_Transmit(eepromBuffer[x]);
	}
}

// Send the ESP8266 the last x items in the log according to MAX_LOG value
void send_log(void) {
	if (logCurrentNo != blankEeprombyte) {
		uint8_t logCounter = logCurrentNo;
		for (uint8_t x = 0; x < MAX_LOG; x++) {
			uint16_t eepromLocation = (logCounter * 64);
			
			// Read i2c EEPROM
			soft_i2c_eeprom_read_x_bytes(EEPROM_ADDR, eepromLocation, 64);
			if (eepromBuffer[0] != blankEeprombyte) {
				for (uint8_t e = 0; e < 64; e++) {
					if (eepromBuffer[e] != 0) {
						USART_Transmit(eepromBuffer[e]);
					}
					else {
						break;
					}
				}
				USART_Transmit(0);
			}
			else {
				break;
			}
			
			if (logCounter == 0) {
				logCounter = (MAX_LOG-1);
			}
			else {
				logCounter--;
			}
			
			_delay_ms(50);
		}	
	}
}


// Send the ESP8266 the email addresses
void send_email_addresses(void) {
	for (uint8_t x = 0; x < 3; x++) {
		uint16_t eepromLocation = ((x * 64) + emailAddressLocation);
		
		// Read i2c EEPROM
		soft_i2c_eeprom_read_x_bytes(EEPROM_ADDR, eepromLocation, 64);
		if (eepromBuffer[0] != blankEeprombyte) {
			USART_Transmit('E');
			USART_Transmit('-');
			USART_Transmit(x+1+48);
			USART_Transmit('-');
			
			for (uint8_t e = 0; e < 64; e++) {
				if (eepromBuffer[e] != 0) {
					USART_Transmit(eepromBuffer[e]);
				}
				else {
					break;
				}
			}
			//USART_Transmit(0);
			USART_Transmit('-');
		}
		else {
			break;
		}
		
		_delay_ms(50);
	}
}


// Read the current RTC time, convert it to normal time format and store to our global variables
void read_rtc_time(void) {
	soft_i2c_master_start_with_8bit_address(RTC_ADDRESS, RTC_RTCSEC, REPEATSTART);
	uint8_t rtcTimeSeconds = soft_i2c_master_read(1);
	soft_i2c_master_stop();
	
	soft_i2c_master_start_with_8bit_address(RTC_ADDRESS, RTC_RTCMIN, REPEATSTART);
	uint8_t rtcTimeMinutes = soft_i2c_master_read(1);
	soft_i2c_master_stop();
	
	soft_i2c_master_start_with_8bit_address(RTC_ADDRESS, RTC_RTCHOUR, REPEATSTART);
	uint8_t rtcTimeHour = soft_i2c_master_read(1);
	soft_i2c_master_stop();
	
	soft_i2c_master_start_with_8bit_address(RTC_ADDRESS, RTC_RTCDATE, REPEATSTART);
	uint8_t rtcTimeDate = soft_i2c_master_read(1);
	soft_i2c_master_stop();
	
	soft_i2c_master_start_with_8bit_address(RTC_ADDRESS, RTC_RTCMTH, REPEATSTART);
	uint8_t rtcTimeMonth = soft_i2c_master_read(1);
	soft_i2c_master_stop();
	
	soft_i2c_master_start_with_8bit_address(RTC_ADDRESS, RTC_RTCYEAR, REPEATSTART);
	uint8_t rtcTimeYear = soft_i2c_master_read(1);
	soft_i2c_master_stop();
	
	// Convert MCP7940M format time to normal time
	rtcAmPm = (rtcTimeHour & 0x20) >> 5;
	rtcSec = (((rtcTimeSeconds & 0x7F) >> 4) * 10) + (rtcTimeSeconds & 0x0F);
	rtcMin = (((rtcTimeMinutes & 0x7F) >> 4) * 10) + (rtcTimeMinutes & 0x0F);
	rtcHr = (((rtcTimeHour & 0x10) >> 4) * 10) + (rtcTimeHour & 0x0F);
	rtcDate = (((rtcTimeDate & 0x30) >> 4) * 10) + (rtcTimeDate & 0x0F);
	rtcMon = (((rtcTimeMonth & 0x10) >> 4) * 10) + (rtcTimeMonth & 0x0F);
	rtcYear = (((rtcTimeYear & 0xF0) >> 4) * 10) + (rtcTimeYear & 0x0F);
}

// Program the RTC time
void program_rtc_time(void) {
	// Change RTC time to MCP7940M format
	if (rtcSec > 10) {
		rtcSec = ((rtcSec / 10) << 4) | (rtcSec % 10);
	}
	rtcSec |= RTC_RTCSEC_ST_BIT; // Set RTC start bit
	if (rtcMin > 10) {
		rtcMin = ((rtcMin / 10) << 4) | (rtcMin % 10);
	}
	if (rtcHr > 10) {
		rtcHr = (1<<4) | (rtcHr % 10);
	}
	rtcHr |= (1<<6) | (rtcAmPm<<5); // Set 12 hour mode and AM/PM bit if needed
	if (rtcDate > 10) {
		rtcDate = ((rtcDate / 10) << 4) | (rtcDate % 10);
	}
	if (rtcMon > 10) {
		rtcMon = (1<<4) | (rtcMon % 10);
	}
	if (rtcYear > 10) {
		rtcYear = ((rtcYear / 10) << 4) | (rtcYear % 10);
	}
	
	// Stop RTC
	soft_i2c_master_start_with_8bit_address(RTC_ADDRESS, RTC_RTCSEC, NOREPEATSTART);
	soft_i2c_master_write(0);
	soft_i2c_master_stop();
	
	// RTC Trim
	soft_i2c_master_start_with_8bit_address(RTC_ADDRESS, RTC_TRIM, NOREPEATSTART);
	soft_i2c_master_write(35);
	soft_i2c_master_stop();
	
	// RTC set clock output at 1Hz
	soft_i2c_master_start_with_8bit_address(RTC_ADDRESS, RTC_CONTROL, NOREPEATSTART);
	soft_i2c_master_write(RTC_CONTROL_SQWEN_BIT);
	soft_i2c_master_stop();
	
	// Re-program RTC time (seconds last as it turns on the RTC)
	soft_i2c_master_start_with_8bit_address(RTC_ADDRESS, RTC_RTCYEAR, NOREPEATSTART);
	soft_i2c_master_write(rtcYear);
	soft_i2c_master_stop();
	soft_i2c_master_start_with_8bit_address(RTC_ADDRESS, RTC_RTCMTH, NOREPEATSTART);
	soft_i2c_master_write(rtcMon);
	soft_i2c_master_stop();
	soft_i2c_master_start_with_8bit_address(RTC_ADDRESS, RTC_RTCDATE, NOREPEATSTART);
	soft_i2c_master_write(rtcDate);
	soft_i2c_master_stop();
	soft_i2c_master_start_with_8bit_address(RTC_ADDRESS, RTC_RTCHOUR, NOREPEATSTART);
	soft_i2c_master_write(rtcHr);
	soft_i2c_master_stop();
	soft_i2c_master_start_with_8bit_address(RTC_ADDRESS, RTC_RTCMIN, NOREPEATSTART);
	soft_i2c_master_write(rtcMin);
	soft_i2c_master_stop();
	soft_i2c_master_start_with_8bit_address(RTC_ADDRESS, RTC_RTCSEC, NOREPEATSTART);
	soft_i2c_master_write(rtcSec);
	soft_i2c_master_stop();
}


// Read the day/time and load it to the eepromBuffer
void datetime_to_buffer(void) {
	read_rtc_time();
	
	char rtcTemp[5];
	
	// Am/Pm to string
	char rtcAmPmStr[3];
	if (rtcAmPm == 0) {
		strncpy(rtcAmPmStr, "AM", 3); 
	}
	else {
		strncpy(rtcAmPmStr, "PM", 3); 
	}
	
	// Second to string with added zero in front if needed
	char rtcSecStr[5];
	itoa(rtcSec, rtcTemp, 10);
	if (rtcSec <= 9) {
		strncpy(rtcSecStr, "0", 2);
		strncat(rtcSecStr, rtcTemp, 3);
	}
	else {
		strncpy(rtcSecStr, rtcTemp, 3);
	}
	
	// Minute to string with added zero in front if needed
	char rtcMinStr[5];
	itoa(rtcMin, rtcTemp, 10);
	if (rtcMin <= 9) {
		strncpy(rtcMinStr, "0", 2);
		strncat(rtcMinStr, rtcTemp, 3);
	}
	else {
		strncpy(rtcMinStr, rtcTemp, 3);
	}
	
	sprintf(eepromBuffer, "%i/%i/20%i %i:%s:%s%s - ", rtcDate, rtcMon, rtcYear, rtcHr, rtcMinStr, rtcSecStr, rtcAmPmStr);
}

// Update sensor info to ESP8266 (i.e S-<sensor>-<zone>-<type>-<name>-<voltage>-) e.g. (i.e S-1-1-PIR-Kitchen-3.5V-)
void update_sensor_info(void) {
	strncpy(usartBuffer, "S-", 3);
	
	// Sensor address
	char fromAddressStr[5];
	itoa(RH_RF22_fromAddress, fromAddressStr, 10);
	strncat(usartBuffer, fromAddressStr, 4);
	strncat(usartBuffer, "-", 2);
	
	// Zone
	soft_i2c_eeprom_read_x_bytes(EEPROM_ADDR, ((RH_RF22_fromAddress-1) * 64) + sensorZoneLocation, 1);
	
	char zoneStr[5];
	itoa(eepromBuffer[0], zoneStr, 10);
	strncat(usartBuffer, zoneStr, 2);
	strncat(usartBuffer, "-", 2);
	
	// Type
	soft_i2c_eeprom_read_x_bytes(EEPROM_ADDR, ((RH_RF22_fromAddress-1) * 64) + sensorTypeLocation, 1);
	
	if (eepromBuffer[0] == PIR_SENSOR) {
		strncat(usartBuffer, "PIR", 11);
	}
	else if (eepromBuffer[0] == DOOR_SENSOR) {
		strncat(usartBuffer, "Door", 11);
	}
	else if (eepromBuffer[0] == SIREN_SENSOR) {
		strncat(usartBuffer, "Siren", 11);
	}
	strncat(usartBuffer, "-", 2);
	
	// Sensor name
	soft_i2c_eeprom_read_x_bytes(EEPROM_ADDR, ((RH_RF22_fromAddress-1) * 64) + sensorNameLocation, 20);
	if (eepromBuffer[0] != blankEeprombyte) {
		strncat(usartBuffer, eepromBuffer, 21);
	}
	else {
		strncat(usartBuffer, "Unnamed", 7);
	}
	strncat(usartBuffer, "-", 2);
	
	// Voltage
	char clientVoltageStr[10];
	double clientVoltage = (double) RH_RF22_rxInfo * (double) 4 * (double) 0.00107421875 * (double) 5.7;
	dtostrf(clientVoltage, 4, 2, clientVoltageStr);
	strncat(usartBuffer, clientVoltageStr, 11);
	strncat(usartBuffer, "V-", 2);
	
	// Transmit
	for (uint8_t x = 0; x < strlen(usartBuffer); x++) {
		USART_Transmit(usartBuffer[x]);
	}
}

// Update sensor verification to ESP8266 (i.e V-1-)
void update_sensor_verification(void) {
	strncpy(usartBuffer, "V-", 3);
	
	// Sensor address
	char fromAddressStr[5];
	itoa(RH_RF22_fromAddress, fromAddressStr, 10);
	strncat(usartBuffer, fromAddressStr, 4);
	strncat(usartBuffer, "-", 2);
	
	// Sensor state
	char sensorStateStr[5];
	itoa(RH_RF22_rxInfo, sensorStateStr, 10);
	strncat(usartBuffer, sensorStateStr, 4);
	strncat(usartBuffer, "-", 2);
	
	// Transmit
	for (uint8_t x = 0; x < strlen(usartBuffer); x++) {
		USART_Transmit(usartBuffer[x]);
	}
}
