/*
 Alarm System v3.0 Server
 Version: Test
 Author: Alex from insideGadgets (http://www.insidegadgets.com)
 Created: 1/04/2017
 Last Modified: 1/06/2017
 
 */
 
#ifndef cbi
#define cbi(sfr, bit) (_SFR_BYTE(sfr) &= ~_BV(bit))
#endif
#ifndef sbi
#define sbi(sfr, bit) (_SFR_BYTE(sfr) |= _BV(bit))
#endif

#ifndef NULL
#define NULL ((void *)0)
#endif

#define LOW 0
#define HIGH 1
#define false 0
#define true 1

// Watchdog timeouts to values
#define T16MS 0
#define T32MS 1
#define T64MS 2
#define T128MS 3
#define T250MS 4
#define T500MS 5
#define T1S 6
#define T2S 7
#define T4S 8
#define T8S 9

// Alarm states
#define SYSTEM_ON 1
#define SYSTEM_OFF 2
#define SYSTEM_ARMED 3
#define SYSTEM_TEST 4
#define SYSTEM_DISABLED 5
#define SIREN_ON 1
#define SIREN_OFF 2

#define ALARM_TRIGGERED 231

#define LED PB0

uint8_t systemState = SYSTEM_OFF;
uint8_t systemStateToSend = 0;
uint8_t sirenState = SIREN_OFF;
uint8_t sensorTypeArmed = 0xFF; // 8 possible sensor types
volatile uint8_t sirenStateCounter = 0;
volatile uint8_t receivedUsart = false;

// Sensor/out types
#define PIR_SENSOR 1
#define DOOR_SENSOR 2
#define SIREN_SENSOR 3

// Si4432
#define SS  PB2
#define si4432_SS_hi	PORTB |=  (1<<SS);
#define si4432_SS_lo	PORTB &= ~(1<<SS);

// Internal EEPROM setting
#define logCurrentNoLocation 1
#define blankEeprombyte 255

// I2C settings
#define TWI_SCL_PIN PC1
#define TWI_SDA_PIN PC0

#define I2C_READ 1
#define I2C_WRITE 0
#define NOREPEATSTART 0
#define REPEATSTART 1
uint8_t I2C_DELAY_CYCLES = 50; // For 12 MHz

// ATSHA204A
#define ATSHA204A_ADDR 0xC8

// External i2c EEPROM settings
#define EEPROM_ADDR 0x50
#define sensorNameLocation 50*64
#define sensorTypeLocation 100*64
#define sensorZoneLocation 150*64
#define emailAddressLocation 200*64

// RTC
#define RTC_ADDRESS 0x6F

#define RTC_RTCSEC 0x00
#define RTC_RTCSEC_ST_BIT 0x80
#define RTC_RTCMIN 0x01
#define RTC_RTCHOUR 0x02
#define RTC_RTCHOUR_TIMEFORMAT_BIT 0x40
#define RTC_RTCDATE 0x04
#define RTC_RTCMTH 0x05
#define RTC_RTCYEAR 0x06

#define RTC_CONTROL 0x07
#define RTC_CONTROL_SQWEN_BIT 0x40
#define RTC_CONTROL_POR_BITS 0x80
#define RTC_TRIM 0x08

uint8_t rtcAmPm = 0;
uint8_t rtcSec = 0;
uint8_t rtcMin = 0;
uint8_t rtcHr = 0;
uint8_t rtcDate = 0;
uint8_t rtcMon = 0;
uint8_t rtcYear = 0;


// Global vars
#define MAX_SENSORS 50
#define MAX_LOG 50

char usartBuffer[65];
char usartRxBuffer[65];
uint8_t usartRxLength = 0;

uint8_t logCurrentNo = 0;
char eepromBuffer[64];

uint8_t dataIn[40];
uint8_t dataBuffer[40];
uint8_t crc[2];
uint8_t randomNumber[32];

// Sensor structure
/*struct Sensor {
	uint8_t zone;
	uint8_t type;
	uint8_t state; // Bit 0 = Alarm armed/On, Bit 1 = Triggered
};
struct Sensor sensorList[MAX_SENSORS];*/

extern uint8_t RH_RF22_init(void);


// Sleep
void system_sleep(void) {
	set_sleep_mode(SLEEP_MODE_PWR_DOWN); // Set sleep mode
	sleep_mode(); // System sleeps here
}

// Setup watchdog timed sequence
void setup_watchdog(uint8_t wdDelay) {
	if (wdDelay > 7) { // Set WDP3 bit
		wdDelay = wdDelay & 7;
		wdDelay |= (1<<5);
	}
	
	MCUSR &= ~(1<<WDRF); // Turn off flag
	WDTCSR |= (1<<WDCE) | (1<<WDE); // Start timed sequence
	WDTCSR = wdDelay | (1<<WDIE); // Set new watchdog timeout value
}

// Turn off the Watchdog
void turn_off_watchdog(void) {
	cli(); // Turn interrupts off
	MCUSR &= ~(1<<WDRF);
	WDTCSR |= (1<<WDCE) | (1<<WDE);
	WDTCSR = 0;
	sei(); // Turn interrupts back on
}

// Watchdog sleep function - setup, sleep and turn off
void watchdog_sleep(uint8_t timeout) { 
	setup_watchdog(timeout);
	system_sleep();
	turn_off_watchdog();
}

// Read a byte from I2C and send Ack if more reads follow else Nak to terminate read
uint8_t soft_i2c_master_read(uint8_t last) {
	uint8_t b = 0;
	for (uint8_t i = 0; i < 8; i++) {
		// Don't change this loop unless you verify the change with a scope
		b <<= 1;
		_delay_loop_1(I2C_DELAY_CYCLES);
		DDRC &= ~(1<<TWI_SCL_PIN);
		_delay_loop_1(I2C_DELAY_CYCLES);
		if (bit_is_set(PINC, TWI_SDA_PIN)) b |= 1;
		_delay_loop_1(I2C_DELAY_CYCLES);
		DDRC |= (1<<TWI_SCL_PIN);
	}
	_delay_loop_1(I2C_DELAY_CYCLES);
	
	// Send Ack or Nak
	if (last) {
		DDRC &= ~(1<<TWI_SDA_PIN);
	}
	else {
		DDRC |= (1<<TWI_SDA_PIN);
	}

	DDRC &= ~(1<<TWI_SCL_PIN);
	_delay_loop_1(I2C_DELAY_CYCLES);
	DDRC |= (1<<TWI_SCL_PIN);
	DDRC &= ~(1<<TWI_SDA_PIN);
	
	return b;
}

// Write a byte to I2C
uint8_t soft_i2c_master_write(uint8_t data) {
	uint8_t rtn = 0;
	
	// Write byte
	for (uint8_t m = 0x80; m != 0; m >>= 1) {
		// Don't change this loop unless you verify the change with a scope
		if (m & data) { 
			DDRC &= ~(1<<TWI_SDA_PIN);
		}
		else {
			DDRC |= (1<<TWI_SDA_PIN);
		}
		
		_delay_loop_1(I2C_DELAY_CYCLES);
		DDRC &= ~(1<<TWI_SCL_PIN);
		_delay_loop_1(I2C_DELAY_CYCLES);
		DDRC |= (1<<TWI_SCL_PIN);
		_delay_loop_1(I2C_DELAY_CYCLES);
	}
	_delay_loop_1(I2C_DELAY_CYCLES);
	
	DDRC &= ~(1<<TWI_SDA_PIN);
	DDRC &= ~(1<<TWI_SCL_PIN);
	_delay_loop_1(I2C_DELAY_CYCLES);
	
	// get Ack or Nak
	rtn = bit_is_set(PINC, TWI_SDA_PIN);
	
	DDRC |= (1<<TWI_SCL_PIN);
	DDRC &= ~(1<<TWI_SDA_PIN);
	_delay_loop_1(I2C_DELAY_CYCLES);
	
	return rtn == 0;
}

// Issue a start condition
uint8_t soft_i2c_master_start(uint8_t addressRW) {
	DDRC |= (1<<TWI_SDA_PIN);
	_delay_loop_1(I2C_DELAY_CYCLES);
	DDRC |= (1<<TWI_SCL_PIN);
	_delay_loop_1(I2C_DELAY_CYCLES);
	
	return soft_i2c_master_write(addressRW);
}

// Issue a restart condition
uint8_t soft_i2c_master_restart(uint8_t addressRW) {
	DDRC &= ~(1<<TWI_SDA_PIN);
	DDRC &= ~(1<<TWI_SCL_PIN);
	_delay_loop_1(I2C_DELAY_CYCLES);
	
	return soft_i2c_master_start(addressRW);
}

// Issue a stop condition
void soft_i2c_master_stop(void) {
	DDRC |= (1<<TWI_SDA_PIN);
	_delay_loop_1(I2C_DELAY_CYCLES);
	
	DDRC &= ~(1<<TWI_SCL_PIN);
	_delay_loop_1(I2C_DELAY_CYCLES);
	
	DDRC &= ~(1<<TWI_SDA_PIN);
	_delay_loop_1(I2C_DELAY_CYCLES);
}

// Issue the device start and the 8bit address to write
uint8_t soft_i2c_master_start_with_8bit_address(uint8_t deviceAddr, uint16_t addressRW, uint8_t repeatedStart) {
	if (!soft_i2c_master_start((deviceAddr<<1) | I2C_WRITE)) return false;
	if (!soft_i2c_master_write(addressRW)) return false;
	if (repeatedStart) { if (!soft_i2c_master_restart((deviceAddr<<1) | I2C_READ)) return false; }
	
	return true;
}

// Issue the device start and the address to write
uint8_t soft_i2c_master_start_with_16bit_address(uint8_t deviceAddr, uint16_t addressRW, uint8_t repeatedStart) {
	if (!soft_i2c_master_start((deviceAddr<<1) | I2C_WRITE)) return false;
	if (!soft_i2c_master_write((addressRW >> 8))) return false; // MSB
	if (!soft_i2c_master_write((addressRW & 0xFF))) return false; // LSB
	if (repeatedStart) { if (!soft_i2c_master_restart((deviceAddr<<1) | I2C_READ)) return false; }
	
	return true;
}

// Read 1 byte from the EEPROM device and return it
uint8_t soft_i2c_eeprom_read_byte(uint8_t deviceAddr, uint16_t readAddress) {
	// Issue start condition, send device address, issue repeated start
	soft_i2c_master_start_with_16bit_address(deviceAddr, readAddress, REPEATSTART);
	
	// Read the byte
	uint8_t byteRead = soft_i2c_master_read(1);
	
	// Issue a stop condition
	soft_i2c_master_stop();
	
	return byteRead;
}

// Write 1 byte to the EEPROM
uint8_t soft_i2c_eeprom_write_byte(uint8_t deviceAddr, uint16_t writeAddress, uint8_t writeByte) {
	// Issue start condition and send device address
	soft_i2c_master_start_with_16bit_address(deviceAddr, writeAddress, NOREPEATSTART);
	
	// Write the byte
	if (!soft_i2c_master_write(writeByte)) return false;
	
	// Issue a stop condition
	soft_i2c_master_stop();
	
	// Delay
	_delay_ms(10);
	
	return true;
}

// Read x bytes from the EEPROM device
uint8_t soft_i2c_eeprom_read_x_bytes(uint8_t deviceAddr, uint16_t readAddress, uint8_t count) {
	// Issue start condition, send device address, issue repeated start
	soft_i2c_master_start_with_16bit_address(deviceAddr, readAddress, REPEATSTART);
	
	// Read the bytes
	uint8_t x = 0;
	if (count > 1) {
		for (x = 0; x < (count-1); x++) {
			eepromBuffer[x] = soft_i2c_master_read(0);
		}
	}
	eepromBuffer[(count-1)] = soft_i2c_master_read(1);
	
	// Issue a stop condition
	soft_i2c_master_stop();
	
	return 1;
}

// Page write 64 bytes to the EEPROM
uint8_t soft_i2c_eeprom_page_write_64_bytes(uint8_t deviceAddr, uint16_t address) {
	
	// Issue start condition and send device address
	soft_i2c_master_start_with_16bit_address(deviceAddr, address, NOREPEATSTART);
	
	// Write all the bytes
	for (uint8_t x = 0; x < 64; x++) {
		soft_i2c_master_write(eepromBuffer[x]);
	}
	
	// Issue a stop condition
	soft_i2c_master_stop();
	
	_delay_ms(10);
	
	return true;
}

// Receive USART data
uint8_t USART_Receive( void ) {
	while ( !(UCSR0A & (1<<RXC0)) ); // Wait for data to be received
	return UDR0; // Get and return received data from buffer
}

// Transmit USART data
void USART_Transmit(unsigned char data) {
	while ( !( UCSR0A & (1<<UDRE0)) ); // Wait for empty transmit buffer
	UDR0 = data;
}

// SPI transfer 1 byte and return the result
uint8_t spi_transfer(uint8_t data) {
	SPDR = data;
	
	// Wait for transmission complete
	while (!(SPSR & (1<<SPIF)));
	
	return SPDR;
}

// Initialise the SPI / SS
void spi_init(void) {
	DDRB |= (1<<PB5); // SPI CLK
	DDRB |= (1<<PB3); // SPI DO
	DDRB &= ~(1<<PB4); // SPI DI
	PORTB |= (1<<PB4); // SPI DI
	
	DDRB |= (1<<SS);
	si4432_SS_hi;
	
	// Enable SPI, Master, set clock rate fck/64
	//SPCR = (1<<SPE)|(1<<MSTR)|(1<<SPR0); // 16
	SPCR = (1<<SPE)|(1<<MSTR)|(1<<SPR1);
}

// Setup
void setup(void) {
	_delay_ms(500); // Wait a little bit after power on
	spi_init(); // Initialise SPI
	
	DDRB |= (1<<LED);
	
	// Initialise RF22
	if (RH_RF22_init()) {
		PORTB |= (1<<LED);
		_delay_ms(50);
		PORTB &= ~(1<<LED);
		_delay_ms(50);
	}
	
	// Initialse I2C
	DDRD &= ~(1<<TWI_SCL_PIN);
	PORTD &= ~(1<<TWI_SCL_PIN);
	DDRD &= ~(1<<TWI_SDA_PIN);
	PORTD &= ~(1<<TWI_SDA_PIN);
	
	// Setup USART0
	UBRR0 = 12; // 115.2k Baud rate
	UCSR0A |= (1<<U2X0); // Double rate
	UCSR0B |= (1<<RXEN0) | (1<<TXEN0); // Receiver/Transmitter enable
	
	// Load log current number
	logCurrentNo = eeprom_read_byte((uint8_t*) logCurrentNoLocation);
	
	// Test sensor name writting to eeprom
	//strncpy(eepromBuffer, "Kitchen", 8);
	//soft_i2c_eeprom_page_write_64_bytes(EEPROM_ADDR, (0 * 64) + sensorNameLocation);
	
	// Timer1 for 5.5s, used for turning off siren after 5 minutes
	TCNT1 = 0; // Reset to 0
	TCCR1B = ((1<<CS12) | (1<<CS10)); // 1024 prescaler
	TIMSK1 |= (1<<TOIE1); // Enable overflow interrupt
	
	sei(); // Turn on interrupts
}
